// ---------------------------------------------------------------------
// File: mathlib.h
// Author: Miguel Torres-Torriti (c) 2008.11.21-2009.02.06 v. 1.0
// Change log:
//	2008.11.22, v.1.0, MTT: First (single-file) version was created
//                          using global variables.
//  2009.02.06, v.2.0, MTT: GUI functions were made to that
//                          the input is passed as a pointer
//                          through the optional parameters of 
//                          the callbacks.
// Description:
//  This file provides basic math definitions and utility functions.
// Remarks:
//  For maximum code efficiency and portability use the mathematical
//  functions in OpenCV or GSL instead of the ones provided here,
//  unless for those convenient definitions and functions that are 
//  not found in OpenCV or GSL.
//  It is also to be noted that some functions provided in mathlib.cpp
//  are intended to illustrate certain basic steps for creating and
//  manipulating matrices.  These routines have a tutorial value, as well
//  as the role of providing the basic functionality to quickly develop
//  and try out computations that often involve vectors and matrices.
//
// ---------------------------------------------------------------------
#ifndef MATHLIB_H
#define MATHLIB_H

#include <math.h>

/* Machine Parameters */
//  Floating Point Number X = m x b^e
//    with m = +-(d1/b+d2/b^2+...dt-1/b^(t-1))  (Sign stored in d0).
//    m: mantissa or fractional part
//    b: base
//    e: exponent
//    t: Digits in the mantissa: d1, d2, ... dt
//    u: Digits in the exponent
//    z: Total number of digits required by the FP data type = t+u
//  Largest Mantissa  (LM) = ((b^t-1)/(b-1))*y0*/b^(t-1) = b-b^(1-t)
//     where y0 = b-1 is the largest digit in base b, e.g. b16 -> y0 = 15d.
//  Largest Exponent   (EMAXs)  = (b^u-1)  // Sign and Magnitude Exponent
//  Largest  Exponent  (EMAXc)  = (b^u-1)-b^u/2
//  Smallest Exponent  (EMINc)  =        -b^u/2+1 = offset + 1
//  Machine Epsilon    (EPSMCH) = b^(1-t)
//  Largest  Magnitude (FPNMAX) = LM*b^EMAX = (b-b^(1-t))*b^EMAX
//  Smallest Magnitude (FPNMIN) = LM*b^EMIN = (b-b^(1-t))*b^EMIN
//  In the case EMAX is s and not c, then FPNMIN = LM*b^(-EMAXs)
// The above expressions are found making use of
//           Sn = sum(k=0:n,r^ky0) = ((r^(n+1)-1)/(r-1))*y0
//
// MACHINE CONSTANTS
//  Intelx86 : Digits Double = t + u = 64 bits
//   b = 2
//   t = 53
//   u = 11
//  Somewhat equivalent
//   b = 16
//   t = 14
//   u = 8.0056... EMAX = 256  EMIN = 256.5
// Machine constants for IEEE machines.
//   b = 2 t = 53 u= 11 offset = b^u/2
#ifndef EPSMCH
#define    EPSMCH      2.22044604926e-16  //2.2204460492503131e-016
#endif

#ifndef MNDBL
#define    MINDBL      2.22507385852e-308 //2.2250738585072014e-308
#endif

#ifndef MAXDBL
#define    MAXDBL      1.79769313485e+308 //1.7976931348623157e+308
#endif
// The above constants can also be found in <include/float.h>...


#ifndef TRUE
#define TRUE    1
#endif
#ifndef FASLE
#define FALSE   0
#endif

#ifndef PI_CONST
#define PI_CONST     3.1415926535897932384626433832795
#endif

#ifndef pi_const
#define pi_const     3.1415926535897932384626433832795
#endif

#ifndef EULER_CONST
#define EULER_CONST  2.71828182845905
#endif

#ifndef MAX
#define MAX(X, Y)       (((X) > (Y)) ? (X) : (Y))
#endif

#ifndef MIN
#define MIN(X, Y)       (((X) < (Y)) ? (X) : (Y))
#endif

// *****************************************************************************
// --- BASIC MATH FUNCTIONS ---
// Miguel Torres Torriti (c) 2008.11.21 v. 1.0
// These functions have been created to illustrate the construction 
// of vectors and matrices.  These functions are improved versions
// of the old Math Library, however it is recommended that the
// versions implemented as part of OpenCV or the GSL libraries
// are used instead for portability.
//

struct mvector{        // mvector structure
	//size_t dt;       // data type (bytes per element)
	unsigned long len; // vector size 
	double *val;	   // vector data values
};

struct mmatrix{         // mmatrix structure
	unsigned long rows; // number of rows
	unsigned long cols; // number of columns
	double **val;	    // matrix data values
};

// Functions to create basic mathematical structures (vectors and matrices).

mvector* mvectoralloc(unsigned long len); // Allocate memory for vector
void mvectorfree(mvector *v);		// Free memory used by the vector
void mvectorprint(mvector *v);		// Print the vector's content
void mvectorfilltest(mvector *v);	// Fill vector with test values (for testing purposes)

mmatrix* mmatrixalloc(unsigned long rows, unsigned long cols); // Allocate memory for matrix
void mmatrixfree(mmatrix *m);		// Free memory used by the matrix
void mmatrixprint(mmatrix *m);		// Print the matrix's content
void mmatrixfilltest(mmatrix *m);	// Fill matrix with test values (for testing purposes)


// *****************************************************************************
// MATH FUNCTIONS OF THE OLD MATH LIBRARY - M.A.TT. 21.VIII.2000
// These functions have been abandoned in favor of similar
// versions implemented as part of OpenCV or the GSL libraries.
//
/*
void *vector(unsigned long nElems, int size);
void freeMat(void **ptr2d);
void **matrix(unsigned long x, unsigned long y, int size);

void copy_matrix(unsigned int rows, unsigned int cols,
                 double **matrix_in, double **matrix_out);

double enorm2(double *vec, unsigned int dimVec); // Squared Euclidean norm
double enorm(double *vec, unsigned int dimVec);  // Euclidean norm

void row_mean(unsigned int rows, unsigned int cols,
              double **vectors, double *mean);

void covar_matrix (int rows, int cols, double **vectors,
		   double *mean, double **covariance);

void matrix_inv (int n, double **matrix_in, double **inv);
void matrix_det (int n, double **matrix_in, double *det);
void matrix_inv_det (int n, double **matrix_in, double **inv, double *det);

void ludcmp(double **a, int n, int *indx, double *d);
void lubksb(double **a, int n, int *indx, double *b);
*/

#endif
